from cfnlint.linter import linter
import click

@click.command()
@click.option('--cfn_file', default='test_stack.yaml', help='The CloudFormation (yaml) file to lint')
@click.option('--custom_ruleset_file', help='The Custom RuleSet file')
@click.option('--aws_region', default='eu-west-1', help='The AWS region')
def main(cfn_file, custom_ruleset_file, aws_region):
    print('cfn-lint: AWS CloudFormation Linter`')
    print('Linting {}'.format(cfn_file))

    errors = linter(cfn_file, custom_ruleset_file)
    print('\n'.join(errors))



import copy
import yaml
import requests


class LintRules(object):

    def __init__(self):
        self.__aws_rules = {}
        self.__custom_rules = {}
        self.__cloudformation_rules = {}

        self.initialize()

    def initialize(self, custom_rules_path=None):
        self._load_aws_rules()

        self.__cloudformation_rules = copy.deepcopy(self.__aws_rules)

        if custom_rules_path:
            self._load_custom_rules(custom_rules_path)
            self._merge_rules()

    def _merge_rules(self):
        aws_resources = self.__aws_rules["ResourceTypes"]
        custom_resources = self.__custom_rules["Resources"]
        cloudformation_resources = self.__cloudformation_rules["ResourceTypes"]

        # Cannot manipulate a dictionary while looping through it, so
        # Use the original AWS rules for this.
        for aws_resource, aws_properties in aws_resources.items():

            # Remove unsupported resources
            if aws_resource not in custom_resources:
                del cloudformation_resources[aws_resource]
                continue

        for cfn_resource, cfn_properties in cloudformation_resources.items():

            # Update mandatory properties
            custom_resource = custom_resources[cfn_resource]
            if not custom_resource:
                continue

            custom_properties = custom_resource["Properties"]
            if not custom_properties:
                continue

            for custom_property, property_values in custom_properties.items():

                cfn_property = cfn_properties["Properties"][custom_property]

                # Override mandatory properties
                cfn_property["Required"] = True

                # Add allowed values property
                if property_values:
                    cfn_property['AllowedValues'] = property_values['Values']

    def _load_aws_rules(self):
        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/cfn-resource-specification.html
        json_request = requests.get('https://d3teyb21fexa9r.cloudfront.net/latest/gzip/CloudFormationResourceSpecification.json')  # noqa: E501
        self.__aws_rules = json_request.json()

    def _load_custom_rules(self, custom_rules_path):
        yaml_file = open(custom_rules_path)
        self.__custom_rules = yaml.load(yaml_file)

    @property
    def cloudformation_rules(self):
        return self.__cloudformation_rules



import os
import yaml
from abc import ABCMeta, abstractmethod

class BaseYamlLinter(metaclass=ABCMeta):

    def __init__(self, file_path, lintingrules=None):

        if not os.path.isfile(file_path):
            raise ValueError("The file '%s' does not exist" % file_path)

        # Global issues
        self.errors = []

        self.file_path = file_path
        self.lintingrules = lintingrules

        self.file_as_yaml = self.__load_yaml_file(file_path)

    def __load_yaml_file(self, file_path):
        with open(file_path) as stream:
            file_contents = stream.read()

        file_contents_yaml = yaml.load(file_contents)

        # Validate the YAML
        self.__validate_valid_yaml(file_path, file_contents_yaml)

        return file_contents_yaml

    def __validate_valid_yaml(self, file_path, file_contents_yaml):
        if file_contents_yaml is None:
            raise ValueError("The file '%s' is empty" % file_path)

        if file_contents_yaml.get('Resources') is None:
            raise ValueError("Mandatory property 'Resources' is missing in file '%s'" % file_path)

    @abstractmethod
    def lint(self):
        """
        Lints the specified yaml file
        """
        pass



import re
import yaml

from collections import OrderedDict
from cfnlint.core.lintrules import LintRules
from cfnlint.core.linter import Linter

def linter(file_path, custom_ruleset=None):
    __setup_yaml_module()

    errors = __lint_file(file_path, custom_ruleset)

    return errors


def __lint_file(file_path, custom_ruleset=None):

    all_errors = []

    # Load ruleset
    lintrules = LintRules()
    lintrules.initialize(custom_ruleset)

    # Lint!
    linter = Linter(file_path, lintrules.cloudformation_rules)
    linter.lint()

    all_errors += linter.errors

    return all_errors


def __setup_yaml_module():
    # preserve order while loading the YAML file by using an OrderedDict instead of a Dict
    # http://stackoverflow.com/a/21048064/3034356
    _mapping_tag = yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG
    yaml.add_constructor(_mapping_tag, __yaml_override_constructor)

    # pyyaml has problems with the CloudFormation functions (such as !Sub)
    # we 'remove' these functions by overriding the constructor again for these functions
    # http://stackoverflow.com/questions/7224033/default-constructor-parameters-in-pyyaml
    yaml.add_constructor('!And', __yaml_override_cloudformation_function)
    yaml.add_constructor('!Base64', __yaml_override_cloudformation_function)
    yaml.add_constructor('!GetAtt', __yaml_override_cloudformation_function)
    yaml.add_constructor('!GetAZs', __yaml_override_cloudformation_function)
    yaml.add_constructor('!FindInMap', __yaml_override_cloudformation_function)
    yaml.add_constructor('!If', __yaml_override_cloudformation_function)
    yaml.add_constructor('!ImportValue', __yaml_override_cloudformation_function)
    yaml.add_constructor('!Join', __yaml_override_cloudformation_function)
    yaml.add_constructor('!Equals', __yaml_override_cloudformation_function)
    yaml.add_constructor('!Ref', __yaml_override_cloudformation_function)
    yaml.add_constructor('!Select', __yaml_override_cloudformation_function)
    yaml.add_constructor('!Split', __yaml_override_cloudformation_function)
    yaml.add_constructor('!Sub', __yaml_override_cloudformation_function)
    yaml.add_constructor('!Not', __yaml_override_cloudformation_function)
    yaml.add_constructor('!Or', __yaml_override_cloudformation_function)


def __yaml_override_constructor(loader, node):
    return OrderedDict(loader.construct_pairs(node))


def __yaml_override_cloudformation_function(loader, node):
    return_value = node.tag + ' '

    if isinstance(node.value, str):
        # If a Cloudformation function is used, the result can be empty. Placeholder variables with "value"
        return_value += re.sub('\${[A-Za-z]*}', 'value', node.value)

    else:
        # convert lists to a meaningful representation of it was about
        return_value += '[' + ', '.join(str(item.value) for item in node.value) + ']'

    return return_value



from cfnlint.core.BaseYamlLinter import BaseYamlLinter


class Linter(BaseYamlLinter):

    def lint(self):
        resources = self.file_as_yaml['Resources']

        lintingrules = self.lintingrules['ResourceTypes']

        for resource_name, resource in resources.items():

            # Skip custom resources, cannot be linted
            if resource['Type'].startswith("Custom::"):
                continue

            # Check if the resource is supported
            if resource['Type'] not in lintingrules:

                self.errors.append(
                    "Unsupported resource: %s (%s)" % (resource_name, resource['Type'])
                )
                continue

            # Check if there are rules specified for the resource
            if not lintingrules[resource['Type']]:
                continue

            if not 'Properties' in resource:
                continue

            # Check mandatory properties
            resource_properties = resource['Properties']

            rule_properties = lintingrules[resource['Type']]['Properties']

            # If there are no required resources, skip
            if not rule_properties:
                continue

            # Validate all properties in the rules
            for rule_property, rules in rule_properties.items():

                if rules['Required']:
                    # Check if the property is specified
                    if rule_property not in resource_properties:
                        self.errors.append(
                            'Mandatory property "{}" not specified for {} ({}). See: {}'
                            .format(rule_property, resource_name, resource['Type'], rules['Documentation'])
                        )
                        continue

                if rule_property in resource_properties:
                    property_value = resource_properties[rule_property]

                    # Check if it has is a valid value
                    if rules:
                        if 'AllowedValues' in rules:
                            if rules['AllowedValues']:
                                if property_value not in rules['AllowedValues']:
                                    self.errors.append(
                                        'Invalid property value "{}" specified for {}.{} See: {}'
                                        .format(property_value, resource_name, rule_property, rules['Documentation'])
                                    )
                                    continue



from setuptools import setup, find_packages

# Custom readme file for pypi
def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='cfnlint',
      version='0.0.9',
      description='AWS Cloudformation linter',
      long_description=readme(),
      classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3.6',
      ],
      keywords='AWS CloudFormation Linter yaml json',
      url='http://github.com/fatbasstard/cfnlinter',
      author='Frank van Boven',
      author_email='frank@cenotaph.nl',
      license='GNU GPL 2.0',
      packages=find_packages(exclude=['tests*']),
      python_requires='>=3',
      install_requires=[
          'click',
          'pyyaml',
          'requests'
      ],
      test_suite='nose.collector',
      tests_require=['nose', 'nose-cover3'],
      entry_points={
          'console_scripts': ['cfn-lint=cfnlint.cli.cli:main'],
      },
      include_package_data=True,
      zip_safe=False)
